# BEGIN COPYRIGHT
# *************************************************************************
# Licensed Materials - Property of IBM 
# 5724-L01, 5655-N53, 5724-I82, 5655-R15
# Copyright IBM Corporation 2008. All rights reserved. 
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
# *************************************************************************
# END COPYRIGHT

import os
import re

#------------------------------------------------------------------------------
# install the application with the specified parameter
#------------------------------------------------------------------------------
def installApplicationES( applicationName, earLocation, cell, node, server ):
	framework.log('info', 'ESBRecoveryUpgrade', 'installApplicationES', 'entering')
	parameters = applicationName + " " + earLocation + " " + cell + " " + node + " " + server
	framework.log('info', 'ESBRecoveryUpgrade', 'installApplicationES', 'inputs are ' + parameters)
	result = 0
	whetherApplicationExist = checkApplicationExist( applicationName )

	#1.IF: the application was not installed yet, install the application 
	if( whetherApplicationExist == 0 ):
		#deployOptions = '-appname ' + applicationName + ' -cell ' +  cell + ' -node ' + node + ' -server ' + server
		deployOptions = ['-appname', applicationName , '-cell', cell, '-node', node, '-server', server]
		dataSourceAttr = ['-DataSourceFor20EJBModules', [['wpsFEMgrEJB', 'wpsFEMgrEJB.jar,META-INF/ejb-jar.xml', 'jdbc/WPSDB']]]
		bindJndiAttr = ['-BindJndiForEJBMessageBinding', [['wpsFEMgrEJB', 'RecoveryMessageDrivenBean', 'wpsFEMgrEJB.jar,META-INF/ejb-jar.xml', [], 'manualrecovery', []]]]
		parameters = deployOptions + dataSourceAttr + bindJndiAttr
		framework.log('info', 'ESBRecoveryUpgrade', 'installApplicationES', parameters)
		try:
			AdminApp.install( earLocation, parameters )
		except:
			framework.log('error', 'ESBRecoveryUpgrade', 'installApplicationES', 'Exception occurred when trying to install application')
			result = 1
			framework.log('error', 'ESBRecoveryUpgrade', 'installApplicationES', 'exiting 0')
			return 0
		else:
			framework.log('info', 'ESBRecoveryUpgrade', 'installApplicationES', 'exiting ' + applicationName + ' installed successfully')
			return 1
			#AdminConfig.save()

	#2.Else: the application was installed, add the mapping if available
	else:
		#2.1 Construct the new target
		newTarget = 'WebSphere:cell=' + cell + ',node=' + node + ',server=' + server

		#2.2 Get the old target information
		moduleList = AdminApp.listModules( applicationName, '-server' )
		infoList = moduleList.split( '#' )
		module = 'wpsFEMgrEJB'
		uri = infoList[1].replace( '+', ',' )
		oldTarget = infoList[2]

		#2.3 check whether the target already exist in the target mapping
		result = checkTargetExist( oldTarget, newTarget )

		#2.3.1 If the target map is already defined, do nothing	
		if( result == 0 ):
			framework.log('info', 'ESBRecoveryUpgrade', 'installApplicationES', 'The application ' + applicationName + ' is already mapped to ' + newTarget)

		#2.3.2 Otherwise, add the target mapping to the server
		else:
			#the moduleList is in format like 
			try:
				detailOption = [[module, uri, oldTarget + '+' + newTarget]]
				options = ['-MapModulesToServers', detailOption]
				AdminApp.edit( applicationName, options )
			except:
				framework.log('error', 'ESBRecoveryUpgrade', 'installApplicationES', 'exiting 0, Exception occurred when mapping to target')
				return 0
			else:
				#AdminConfig.save()
				framework.log('info', 'ESBRecoveryUpgrade', 'installApplicationES', 'exiting 1, application mapping successful')
				return 1
	framework.log('info', 'ESBRecoveryUpgrade', 'installApplicationES', 'exiting')


#------------------------------------------------------------------------------
# install the application with the specified parameter
#------------------------------------------------------------------------------
def installApplicationClusterES( applicationName, earLocation, cell, cluster ):
	framework.log('info', 'ESBRecoveryUpgrade', 'installApplicationClusterES', 'Entering installApplicationClusterES')
	parameters = applicationName + ' ' + earLocation + ' ' + cell + ' ' + cluster
	framework.log('info', 'ESBRecoveryUpgrade', 'installApplicationClusterES', parameters)

	result = 0
	whetherApplicationExist = checkApplicationExist( applicationName )

	#1.IF: the application was not installed yet, install the application 
	if( whetherApplicationExist == 0 ):
		deployOptions = ['-appname', applicationName , '-cell', cell, '-cluster', cluster]
		dataSourceAttr = ['-DataSourceFor20EJBModules', [['wpsFEMgrEJB', 'wpsFEMgrEJB.jar,META-INF/ejb-jar.xml', 'jdbc/WPSDB']]]
		bindJndiAttr = ['-BindJndiForEJBMessageBinding', [['wpsFEMgrEJB', 'RecoveryMessageDrivenBean', 'wpsFEMgrEJB.jar,META-INF/ejb-jar.xml', [], 'manualrecovery', []]]]
		parameters = deployOptions + dataSourceAttr + bindJndiAttr
		try:
			AdminApp.install( earLocation, parameters )
		except:
			framework.log('error', 'ESBRecoveryUpgrade', 'installApplicationClusterES', 'exiting 0, Exception occurred when installing application')
			return 0
		else:
			#AdminConfig.save()
			framework.log('info', 'ESBRecoveryUpgrade', 'installApplicationClusterES', 'exiting 1, Install application successful')
			return 1

	#2.Else: the application was installed, add the mapping if available
	else:
		# 2.1 Construct the new target
		newTarget = 'WebSphere:cell=' + cell + ',cluster=' + cluster

		# 2.2 Get the old target information
		moduleList = AdminApp.listModules( applicationName, '-server' )
		infoList = moduleList.split( '#' )
		module = 'wpsFEMgrEJB'
		uri = infoList[1].replace( '+', ',' )
		oldTarget = infoList[2]

		#2.3 check whether the target already exist in the target mapping
		result = checkTargetExist( oldTarget, newTarget )

		#2.3.1 If the target map is already defined, do nothing	
		if( result == 0 ):
			framework.log('info', 'ESBRecoveryUpgrade', 'installApplicationClusterES', 'Application is already mapped to target')

		#2.3.2 Otherwise, add the target mapping to the server
		else:
			try:
				detailOption = [[module, uri, oldTarget + '+' + newTarget]]
				options = ['-MapModulesToServers', detailOption]
				AdminApp.edit( applicationName, options )
			except:
				framework.log('error', 'ESBRecoveryUpgrade', 'installApplicationClusterES', 'Exiting 0, exception happened when trying to add the target mapping')				
				return 0
			else:				
				framework.log('info', 'ESBRecoveryUpgrade', 'installApplicationClusterES', 'Exiting 1, adding target mapping successful')
				return 1
	framework.log('info', 'ESBRecoveryUpgrade', 'installApplicationClusterES', 'exiting ' + str(result))
	return result;




#------------------------------------------------------------------------------
# get the location of installable RemoteAL application
#------------------------------------------------------------------------------	
def getRecoveryApplicationLocationES( cell, node ):
	framework.log('info', 'ESBRecoveryUpgrade', 'getRecoveryApplicationLocationES', 'Entering ' + cell + ' ' + node)

	ret = getWASHome() + '/installableApps/wpsFEMgr_esb.ear'

	framework.log('info', 'ESBRecoveryUpgrade', 'getRecoveryApplicationLocationES', 'Exiting ' + ret)
	return ret





#------------------------------------------------------------------------------
def findDestination( scabusName, queueId ):
	framework.log('info', 'ESBRecoveryUpgrade', 'findDestination', 'Entering ' + scabusName + ' ' + queueId)

	busList = AdminConfig.list( 'SIBus' )
	busList = wsadminToList( busList )
	for bus in busList:
		busName = AdminConfig.showAttribute( bus, 'name' )
		if( busName == scabusName ):
			destinationList = AdminConfig.list( 'SIBQueue', bus )
			destinationList = wsadminToList( destinationList )
			for destination in destinationList:
				destinationID = AdminConfig.showAttribute( destination, 'identifier' )
				if( destinationID == queueId ):
					framework.log('info', 'ESBRecoveryUpgrade', 'findDestination',	'The destination ' + destinationID + ' already exists')
					return destination
	framework.log('info', 'ESBRecoveryUpgrade', 'findDestination',	'exiting ')


def createSIBDestination( busName, nodeName, serverName, clusterName, destName ):
	framework.log('info', 'ESBRecoveryUpgrade', 'createSIBDestination', 'Entering')
	parameters = busName + ' ' + nodeName + ' ' + serverName + ' ' + clusterName + ' ' + destName
	framework.log('info', 'ESBRecoveryUpgrade', 'createSIBDestination', parameters)
	
	if( clusterName == '' ):
		#destName = 'WBI.FailedEvent.'+nodeName+'.'+ serverName
		try:
			#print  '[-bus ' + busName + ' -name ' + destName + ' -type QUEUE ' + '-node ' + nodeName + '-server ' + serverName + ']'
			AdminTask.createSIBDestination( '[-bus ' + busName + ' -name ' + destName + ' -type QUEUE ' + '-node ' + nodeName + ' -server ' + serverName + ']' )
		except:
			framework.log('error', 'ESBRecoveryUpgrade', 'createSIBDestination', 'Exiting 0, error cocurred when trying to createSIBDestination')
			return 0
		else:
			#AdminConfig.save()
			framework.log('info', 'ESBRecoveryUpgrade', 'createSIBDestination', 'Exiting 1, create SIBDestination successful')
			return 1
	else:
		#destName = 'WBI.FailedEvent.'+clusterName
		try:
			#print  '[-bus ' + busName + ' -name ' + destName + ' -type QUEUE' + ' -cluster ' + clusterName + ']'
			AdminTask.createSIBDestination( '[-bus ' + busName + ' -name ' + destName + ' -type QUEUE' + ' -cluster ' + clusterName + ']' )
		except:
			framework.log('error', 'ESBRecoveryUpgrade', 'createSIBDestination', 'Exiting 0, error cocurred when trying to createSIBDestination')
#			return 0
# TODO for some reason, the AppCluster queues seem to already exist, and it's causing failures here
			framework.log('error', 'ESBRecoveryUpgrade', 'createSIBDestination', 'Exiting 1, attempting to continue')
			return 1
		else:
			#AdminConfig.save()
			framework.log('info', 'ESBRecoveryUpgrade', 'createSIBDestination', 'Exiting 1, create SIBDestination successful')
			return 1
	framework.log('info', 'ESBRecoveryUpgrade', 'createSIBDestination', 'Exiting')

def lookupMELocation( nodeName, serverName, clusterName ):
	framework.log('info', 'ESBRecoveryUpgrade', 'lookupMELocation', 'Entering')
	parameters = nodeName + ' ' + serverName + ' ' + clusterName
	framework.log('info', 'ESBRecoveryUpgrade', 'lookupMELocation', parameters)

	if( clusterName == '' ):
		cell = AdminConfig.list( 'Cell' )
		nodeList = AdminConfig.list( 'Node', cell )
		nodeList = wsadminToList( nodeList )
		for node in nodeList:
			currentNodeName = AdminConfig.showAttribute( node, 'name' )
			if( currentNodeName == nodeName ):
				serverList = AdminConfig.list( 'Server', node )
				serverList = wsadminToList( serverList )
				for server in serverList:
					currentServerName = AdminConfig.showAttribute( server, 'name' )
					framework.log('info', 'ESBRecoveryUpgrade', 'lookupMELocation','currentServerName is: ' + currentServerName)
					if( currentServerName == serverName ):
						scopelist = [server, node, cell]
						for scope in scopelist:
							variableMAPList = AdminConfig.list( 'VariableMap', scope )
							variableMAPList = wsadminToList( variableMAPList )
							variableMap = variableMAPList[0]
							entryList = AdminConfig.showAttribute( variableMap, 'entries' )
							entryList = wsadminToList( entryList )
							for entry in entryList:
								symbolicName = AdminConfig.showAttribute( entry, 'symbolicName' )
								if( symbolicName == 'SCA_REMOTE_DESTINATION_LOCATION' ):
									value = AdminConfig.showAttribute( entry, 'value' )
									framework.log('info', 'ESBRecoveryUpgrade', 'lookupMELocation', 'Exiting lookupMELocation ' + str(value))
									return value
		framework.log('info', 'ESBRecoveryUpgrade', 'lookupMELocation', 'Exiting')
		return ''
	else:
		cell = AdminConfig.list( 'Cell' )
		clusters = AdminConfig.list( 'ServerCluster' )
		clusters = wsadminToList( clusters )
		framework.log('info', 'ESBRecoveryUpgrade', 'lookupMELocation', 'Clusters is ' + ''.join(clusters))
		for cluster in clusters:
			currentClusterName = AdminConfig.showAttribute( cluster, 'name' )
			framework.log('info', 'ESBRecoveryUpgrade', 'lookupMELocation', 'CurrentClusterName is ' + currentClusterName)
			if( currentClusterName == clusterName ):
				variableMAPList = AdminConfig.list( 'VariableMap', cluster )
				variableMAPList = wsadminToList( variableMAPList )
				variableMap = variableMAPList[0]
				entryList = AdminConfig.showAttribute( variableMap, 'entries' )
				entryList = wsadminToList( entryList )
				for entry in entryList:
					symbolicName = AdminConfig.showAttribute( entry, 'symbolicName' )
					if( symbolicName == 'SCA_REMOTE_DESTINATION_LOCATION' ):
						value = AdminConfig.showAttribute( entry, 'value' )
						framework.log('info', 'ESBRecoveryUpgrade', 'lookupMELocation', 'Exiting ' + str(value))
						return value
		framework.log('info', 'ESBRecoveryUpgrade', 'lookupMELocation', 'Exiting')
		return ''
	framework.log('info', 'ESBRecoveryUpgrade', 'lookupMELocation', 'Exiting')

def createASForStandAlone( activationSpecName, activationSpecJndiName, messageListenerType, busName, destinationName, messageSelector ):
	framework.log('info', 'ESBRecoveryUpgrade', 'createASForStandAlone', 'Entering')
	parameters = messageListenerType + ' ' + busName + ' ' + destinationName + ' ' + messageSelector
	framework.log('info', 'ESBRecoveryUpgrade', 'createASForStandAlone', parameters)

	if( findASForStandAlone() != 1 ):
		aliasName = 'SCA_Auth_Alias';
		resourceAdapterList = AdminConfig.list( 'J2CResourceAdapter' )
		resourceAdapterList = wsadminToList( resourceAdapterList )
		resourceAdapter = resourceAdapterList[0]
		framework.log('info', 'ESBRecoveryUpgrade', 'createASForStandAlone', 'Resource adapter is ' + resourceAdapter)
		params = '[-name ' + activationSpecName + ' -jndiName ' + activationSpecJndiName + ' -messageListenerType ' + messageListenerType + ' -authenticationAlias ' + aliasName + ']'
		framework.log('info', 'ESBRecoveryUpgrade', 'createASForStandAlone', 'parameters ' + params)
		try:
			AdminTask.createJ2CActivationSpec( resourceAdapter, params )
			framework.log('info', 'ESBRecoveryUpgrade', 'createASForStandAlone', 'J2CAS created successfully')
			destinationTypeId = AdminConfig.getid ( '/J2CActivationSpec:' + activationSpecName + '/J2EEResourceProperty:destinationType/' )
			framework.log('info', 'ESBRecoveryUpgrade', 'createASForStandAlone', 'destinationTypeId ' + destinationTypeId)
			AdminConfig.modify( destinationTypeId, [['value', 'Queue']] )  #need to check
			destinationNameId = AdminConfig.getid( '/J2CActivationSpec:' + activationSpecName + '/J2EEResourceProperty:destinationName/' )
			AdminConfig.modify( destinationNameId, [['value', destinationName]] )
			framework.log('info', 'ESBRecoveryUpgrade', 'createASForStandAlone', 'destinationNameId ' + destinationNameId)
			busNameId = AdminConfig.getid( '/J2CActivationSpec:' + activationSpecName + '/J2EEResourceProperty:busName/' )

			framework.log('info', 'ESBRecoveryUpgrade', 'createASForStandAlone', 'busNameId ' + busNameId)
			AdminConfig.modify( busNameId, [['value', busName]] )

			messageSelectorID = AdminConfig.getid( '/J2CActivationSpec:' + activationSpecName + '/J2EEResourceProperty:messageSelector/' )
			AdminConfig.modify( messageSelectorID, [['value', messageSelector]] )
		except:
			framework.log('error', 'ESBRecoveryUpgrade', 'createASForStandAlone', 'Exiting 0, create AS for standalone failed')
			return 0
		else:
			#AdminConfig.save()
			framework.log('info', 'ESBRecoveryUpgrade', 'createASForStandAlone', 'Exiting 1, created AS for StandAlone')
			return 1
	framework.log('info', 'ESBRecoveryUpgrade', 'createASForStandAlone', 'Exiting 1')
	return 1

def createASForNDServer( activationSpecName, activationSpecJndiName, messageListenerType, busName, destinationName, messageSelector, nodeName, serverName, clusterName ):
	framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Entering')
	framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', nodeName + ' ' + serverName + ' ' + clusterName)

	aliasName = 'SCA_Auth_Alias';
	if( clusterName == '' ):
		cell = AdminConfig.list( 'Cell' )
		nodeList = AdminConfig.list( 'Node', cell )
		nodeList = wsadminToList( nodeList )
		for node in nodeList:
			currentNodeName = AdminConfig.showAttribute( node, 'name' )
			if( currentNodeName == nodeName ):
				serverList = AdminConfig.list( 'Server', node )
				serverList = wsadminToList( serverList )
				for server in serverList:
					currentServerName = AdminConfig.showAttribute( server, 'name' )
					framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Current server ' + currentServerName)
					if( currentServerName == serverName ):
						if( findAS( server, '' ) != 1 ):
							framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'The AS does not exist')
							ralist = AdminConfig.list( 'J2CResourceAdapter', server )
							ralist = wsadminToList( ralist )
							framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Resource adapters on server' + ''.join(ralist))
							if( ralist == [] ):
								framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Exiting 0, J2CResourceAdapter does not exist on the server')								
								return 0
							else:
								for J2Cra in ralist:
									resourceAdapterList = AdminConfig.list( 'ResourceAdapter', J2Cra )
									resourceAdapterList = wsadminToList( resourceAdapterList )
									framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Resource adapters ' + ''.join(resourceAdapterList))
									for ra in resourceAdapterList:
										raclass = AdminConfig.showAttribute( ra, 'resourceAdapterClass' )
										if( raclass == 'com.ibm.ws.sib.ra.inbound.impl.SibRaResourceAdapterImpl' ):
											params = '[-name ' + activationSpecName + ' -jndiName ' + activationSpecJndiName + ' -messageListenerType ' + messageListenerType + ' -authenticationAlias ' + aliasName + ']'
											framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Params is ' + params)
											try:
												J2CASpecID = AdminTask.createJ2CActivationSpec( J2Cra, params )
												framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'AS created successfully')
												update( J2CASpecID, busName, destinationName, messageSelector )
												framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Updated AS attribute successfully')
											except:
												framework.log('error', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Exiting 0, create AS or update AS attribute failed')												
												return 0
											else:
												framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Exiting 1')
												return 1
						else:
							framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Exiting 1, the AS exists on the server')							
							return 1
		framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Exiting 0')
		return 0
	else:
		clusterID = AdminConfig.getid( '/ServerCluster:' + clusterName )
		framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'clusterId is ' + clusterID)
		if( findAS( '', clusterID ) != 1 ):
			ralist = AdminConfig.list( 'J2CResourceAdapter', clusterID )
			ralist = wsadminToList( ralist )
			if( ralist == [] ):
				framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Exiting the J2CResourceAdapter does not existed on the cluster')				
				return
			else:
				for J2Cra in ralist:
					resourceAdapterList = AdminConfig.list( 'ResourceAdapter', J2Cra )
					resourceAdapterList = wsadminToList( resourceAdapterList )
					framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', ''.join(resourceAdapterList))
					for ra in resourceAdapterList:
						raclass = AdminConfig.showAttribute( ra, 'resourceAdapterClass' )
						if( raclass == 'com.ibm.ws.sib.ra.inbound.impl.SibRaResourceAdapterImpl' ):
							params = '[-name ' + activationSpecName + ' -jndiName ' + activationSpecJndiName + ' -messageListenerType ' + messageListenerType + ' -authenticationAlias ' + aliasName + ']'
							try:
								J2CASpecID = AdminTask.createJ2CActivationSpec( J2Cra, params )
								framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Created AS successfully')
								update( J2CASpecID, busName, destinationName, messageSelector )
								framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer','Updated AS successfully')
							except:
								framework.log('error', 'ESBRecoveryUpgrade', 'createASForNDServer',' Exiting 0, create AS or update AS attribute failed')
								return 0
							else:
								framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Exiting 1')
								return 1
		else:
			framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Exiting 1, the AS exists on the target cluster')
			return 1
		framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Exiting 0')
		return 0
	framework.log('info', 'ESBRecoveryUpgrade', 'createASForNDServer', 'Exiting')

def update( J2CASpecID, busName, destinationName, messageSelector ):
	framework.log('info', 'ESBRecoveryUpgrade', 'update', 'Entering')
	parameters = J2CASpecID + ' ' + busName + ' ' + destinationName + ' ' + messageSelector
	framework.log('info', 'ESBRecoveryUpgrade', 'update', parameters)

	j2eeResouceProperties = AdminConfig.showAttribute( J2CASpecID, 'resourceProperties' )
	j2eeResouceProperties = wsadminToList( j2eeResouceProperties )
	for j2eeResouceProperty in j2eeResouceProperties:
		name = AdminConfig.showAttribute( j2eeResouceProperty, 'name' )
		if( name == 'destinationType' ):
			AdminConfig.modify( j2eeResouceProperty, [['value', 'Queue' ]] )
		elif( name == 'destinationName' ):
			AdminConfig.modify( j2eeResouceProperty, [['value', destinationName ]] )
		elif( name == 'busName' ):
			AdminConfig.modify( j2eeResouceProperty, [['value', busName ]] )
		elif( name == 'messageSelector' ):
			AdminConfig.modify( j2eeResouceProperty, [['value', messageSelector ]] )
	framework.log('info', 'ESBRecoveryUpgrade', 'update', 'Exiting')

def findAS( serverID, clusterID ):
	framework.log('info', 'ESBRecoveryUpgrade', 'findAS', 'Entering ' + serverID + ' ' + clusterID)

	if( clusterID == '' ):
		asList = AdminConfig.list( 'J2CActivationSpec', serverID )
		asList = wsadminToList( asList )
		for a in asList:
			name = AdminConfig.showAttribute( a, 'name' )
			if( name == 'failedevent_AS' ):
				framework.log('info', 'ESBRecoveryUpgrade', 'findAS', 'Exiting 1, found AS')
				return 1
		framework.log('info', 'ESBRecoveryUpgrade', 'findAS', 'Exiting 0')
		return 0
	else:
		asList = AdminConfig.list( 'J2CActivationSpec', clusterID )
		asList = wsadminToList( asList )
		for a in asList:
			name = AdminConfig.showAttribute( a, 'name' )
			if( name == 'failedevent_AS' ):
				framework.log('info', 'ESBRecoveryUpgrade', 'findAS', 'Exiting 1, found AS')
				return 1
		framework.log('info', 'ESBRecoveryUpgrade', 'findAS', 'Exiting 0')
		return 0
	framework.log('info', 'ESBRecoveryUpgrade', 'findAS', 'Exiting')

def findASForStandAlone():
	framework.log('info', 'ESBRecoveryUpgrade', 'findASForStandAlone', 'Entering')
	asID = AdminConfig.list( 'J2CActivationSpec' )
	asID = wsadminToList( asID )
	for a in asID:
		name = AdminConfig.showAttribute( a, 'name' )
		if( name == 'failedevent_AS' ):
			framework.log('info', 'ESBRecoveryUpgrade', 'findASForStandAlone', 'Exiting 1, found AS')
			return 1
	framework.log('info', 'ESBRecoveryUpgrade', 'findASForStandAlone', 'Exiting 0')
	return 0



def updateAppDestinations( cellName, nodeName, serverName, clusterName, meClusterName ):
	framework.log('info', 'ESBRecoveryUpgrade', 'updateAppDestinations', 'Entering')
	parameters = cellName + ' ' + nodeName + ' ' + serverName + ' ' +  clusterName + ' ' + meClusterName
	framework.log('info', 'ESBRecoveryUpgrade', 'updateAppDestinations', parameters)

	busList = AdminConfig.list( 'SIBus' )
	busList = wsadminToList( busList )
	for bus in busList:
		busName = AdminConfig.showAttribute( bus, 'name' )
		framework.log('info', 'ESBRecoveryUpgrade', 'updateAppDestinations', 'Bus name is ' + busName)
		scaSystemBusName = 'SCA.SYSTEM.' + cellName + '.Bus'
		#print  'updateAppDestinations. busName = ', busName
		#print  'updateAppDestinations. scaSystemBusName = ', scaSystemBusName
		if( busName == scaSystemBusName ):
			# here we know we're working on the correct bus (SYSTEM)
			#print 'updateAppDestinations:  bus names equal'
			destinationList = AdminConfig.list( 'SIBQueue', bus )
			destinationList = wsadminToList( destinationList )
			for destination in destinationList:
				localpointsRef = AdminConfig.showAttribute( destination, 'localizationPointRefs' )
				localpointsRef = wsadminToList( localpointsRef )
				localpointsRef = localpointsRef[0]
				#print 'updateAppDestinations.   localpointsRef = ', localpointsRef
				currentNodeName = AdminConfig.showAttribute( localpointsRef, 'node' )
				currentServerName = AdminConfig.showAttribute( localpointsRef, 'server' )
				currentClusterName = AdminConfig.showAttribute( localpointsRef, 'cluster' )

				# each |destination| is a queue, the name of which is |identifier|
				identifier = AdminConfig.showAttribute( destination, 'identifier' )
				#print 'updateAppDestinations.   identifier = ', identifier
				if( identifier.startswith( 'sca/' ) ):
					#print 'updateAppDestinations:	identifier starts with sca/'
					if( ( currentNodeName == nodeName ) and ( currentServerName == serverName ) ):
						#print 'updateAppDestinations:	 node and server names equal'
						try:
							#print "updateAppDestinations:	  AdminConfig.modify(", identifier, ", [[\'exceptionDestination\', \'WBI.FailedEvent.\' + ", nodeName, " + \'.\' + ", serverName, "]])"
							AdminConfig.modify( destination, [['exceptionDestination', 'WBI.FailedEvent.' + nodeName + '.' + serverName]] )
						except:
							framework.log('error', 'ESBRecoveryUpgrade', 'updateAppDestinations', 'Exiting 0, modify app destination failed')
							return 0
						else:
							framework.log('info', 'ESBRecoveryUpgrade', 'updateAppDestinations', 'Exiting 1, modified app destination successfully')
							return 1
					appList = AdminConfig.list( 'SCAApplicationModule' )
					appList = wsadminToList( appList )
					for app in appList:
						# each |app| is a specifier for a single application. we can extract its "name", the actual module name, and the "applicationName", the identifier we chose upon installation
						appQName = AdminConfig.showAttribute( app, 'name' )
						appName = AdminConfig.showAttribute( app, 'applicationName' )
						#print 'updateAppDestinations.	 app = ', app
						#print 'updateAppDestinations.	 appName = ', appName
						#print 'updateAppDestinations.	 appQName = ', appQName

						# now build a list of Deployments
						depList = AdminConfig.list( 'Deployment' )
						depList = wsadminToList( depList )
						for dep in depList:
							depName = dep.split( '(' )[0]
							#print 'updateAppDestinations.	  dep = ', dep
							#print 'updateAppDestinations.	  depName = ', depName
							# find the Deployment that matches the app we're looking at
							if ( depName == appName ):
								if ( identifier.startswith( 'sca/' + appQName ) ):
									#print 'updateAppDestinations:		identifier starts with sca/' + appQName
									# here, |dep| matches |destination| (queue)
									targetClusterList = AdminConfig.showAttribute( dep, 'deploymentTargets' )
									targetClusterList = wsadminToList( targetClusterList )
									for targetCluster in targetClusterList:
										# each |targetCluster| is a cluster on which |dep| is installed, and its name is |targetClusterName|
										targetClusterName = AdminConfig.showAttribute( targetCluster, 'name' )
										#print 'updateAppDestinations.		 target = ', targetCluster
										#print 'updateAppDestinations.		 targetClusterName = ', targetClusterName
										if ( targetClusterName == clusterName ):
											# here, the cluster on which |application| is installed matches the cluster we're supposed to be updating
											#print "updateAppDestinations:		  targetClusterName == clusterName"
											#print  'updateAppDestinations.		  currentNodeName = ', currentNodeName
											#print  'updateAppDestinations.		  currentServerName = ', currentServerName
											#print  'updateAppDestinations.		  currentClusterName = ', currentClusterName
											if( currentClusterName == meClusterName ):
												#print 'updateAppDestinations:		   cluster names equal'
												try:
													framework.log('info', 'ESBRecoveryUpgrade', 'updateAppDestinations', 'Cluster is ' + clusterName)
													AdminConfig.modify( destination, [['exceptionDestination', 'WBI.FailedEvent.' + clusterName]] )
												except:
													framework.log('error', 'ESBRecoveryUpgrade', 'updateAppDestinations', 'Exiting 0, modify app destination failed')
													return 0
												else:
													framework.log('info', 'ESBRecoveryUpgrade', 'updateAppDestinations', 'Exiting 1, modified app destination successfully')
													return 1
	framework.log('info', 'ESBRecoveryUpgrade', 'updateAppDestinations', 'Exiting 1')
	return 1



def isSCAConfigured( nodeName, serverName, clusterName ):
	framework.log('info', 'ESBRecoveryUpgrade', 'isSCAConfigured', 'Entering ' + nodeName + ' ' +serverName + ' ' + clusterName)

	if( clusterName == '' ):
		try:
			configList = AdminTask.showSCAAsyncConfig( '[-nodeName ' + nodeName + ' -serverName ' + serverName + ']' )
		except:
			framework.log('info', 'ESBRecoveryUpgrade', 'isSCAConfigured', 'Exiting 0, SCA is not configured on server')
			return 0
		else:
			if( configList.find( '[configured true]' ) > 0 ):
				framework.log('info', 'ESBRecoveryUpgrade', 'isSCAConfigured', 'Exiting 1')
				return 1
			else:
				framework.log('info', 'ESBRecoveryUpgrade', 'isSCAConfigured', 'Exiting 0, SCA is not configured on server')
				return 0
	else:
		try:
			configList = AdminTask.showSCAAsyncConfig( '[-clusterName ' + clusterName + ']' )
		except:
			framework.log('error', 'ESBRecoveryUpgrade', 'isSCAConfigured', 'Exiting 0, SCA is not configured on server')
			return 0
		else:
			if( configList.find( '[configured true]' ) > 0 ):
				framework.log('info', 'ESBRecoveryUpgrade', 'isSCAConfigured', 'Exiting 1')
				return 1
			else:
				framework.log('info', 'ESBRecoveryUpgrade', 'isSCAConfigured', 'Exiting 0')
				return 0
	framework.log('info', 'ESBRecoveryUpgrade', 'isSCAConfigured', 'Exiting')

def parseNode( remoteME ):
	framework.log('info', 'ESBRecoveryUpgrade', 'parseNode', 'Entering ' + remoteME)
	
	if( remoteME.find( 'node=' ) != -1 ) and ( remoteME.find( 'server=' ) != -1 ):
		begin = remoteME.find( 'node=' )
		begin = begin + len( 'node=' )
		end = remoteME.find( ',server=' )
		node = remoteME[begin:end]
		framework.log('info', 'ESBRecoveryUpgrade', 'parseNode', 'Exiting ' + node)
		return node
	else:
		framework.log('info', 'ESBRecoveryUpgrade', 'parseNode', 'Exiting')
		return ''

def parseServer( remoteME ):
	framework.log('info', 'ESBRecoveryUpgrade', 'parseServer', 'Entering ' + remoteME)
	if( remoteME.find( 'node=' ) != -1 ) and ( remoteME.find( 'server=' ) != -1 ):
		begin = remoteME.find( 'server=' )
		begin = begin + len( 'server=' )
		server = remoteME[begin:len( remoteME )]
		framework.log('info', 'ESBRecoveryUpgrade', 'parseServer', 'Exiting ' + server)
		return server
	else:
		framework.log('info', 'ESBRecoveryUpgrade', 'parseServer', 'Exiting')
		return ''

def parseCluster( remoteME ):
	framework.log('info', 'ESBRecoveryUpgrade', 'parseCluster', 'Entering ' + remoteME)
	
	if( remoteME.find( 'cluster=' ) != -1 ):
		cluster = remoteME[18:len( remoteME )]
		framework.log('info', 'ESBRecoveryUpgrade', 'parseCluster', 'Exiting ' + cluster)
		return cluster
	else:
		framework.log('info', 'ESBRecoveryUpgrade', 'parseClsuter', 'Exiting')
		return ''

# returns "1" if and only if |cluster| is the messaging cluster for the deployment environment
def isMessagingCluster ( cluster ):
	framework.log('info', 'ESBRecoveryUpgrade', 'isMessagingCluster', 'Entering ' + cluster)
	
	# this returns the cell of the dmgr to which the script is connected
	cell = AdminConfig.list( 'Cell' )
	framework.log('info', 'ESBRecoveryUpgrade', 'isMessagingCluster', 'Cell is ' + cell)
	wsadminToList( cell )
	# cell-scope search
	variableMapList = AdminConfig.list( 'VariableMap', cell )
	variableMapList = wsadminToList( variableMapList )
	for variableMap in variableMapList:
		entryList = AdminConfig.showAttribute( variableMap, 'entries' )
		entryList = wsadminToList( entryList )
		for entry in entryList:
			symbolicName = AdminConfig.showAttribute( entry, 'symbolicName' )
			if ( symbolicName == "SCA_REMOTE_DESTINATION_LOCATION" ):
				messagingCluster = AdminConfig.showAttribute( entry, 'value' )
				framework.log('info', 'ESBRecoveryUpgrade', 'isMessagingCluster', 'Messaging cluster ' + messagingCluster)
				if ( messagingCluster == str( "WebSphere:cluster=" + cluster ) ):
					framework.log('info', 'ESBRecoveryUpgrade', 'isMessagingCluster', 'Exiting 1, this is messaging cluster')
					return 1
				else:
					# we know we're not the messaging cluster
					framework.log('info', 'ESBRecoveryUpgrade', 'isMessagingCluster', 'Exiting 0, this is not messaging cluster')
					return 0
	# assume the lack of an appropriate variable means there is no separate messaging cluster
	framework.log('info', 'ESBRecoveryUpgrade', 'isMessagingCluster', 'Exiting 0, this is not messaging cluster')
	return 0

#------------------------------------------------------------------------------
# Main entry point
#	1.	Convert Recovery Data 
# 2. Read the properties to decide whether Recovery application need to updated
#------------------------------------------------------------------------------
def upgradeESBProfile():
	framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Entering')
	# 1. check whether the update procedure should be invoked
	whetherUpdateRequired = checkWhetherUpdateRequired();

	# 2. If update required, perform update
	if ( whetherUpdateRequired == 0 ):
		#2.1 Initialization
		recoveryApplicationName = 'wpsFEMgr_' + getWBIVersion()
		framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Reocvery application name is ' + recoveryApplicationName)
		cell = getCell()
		node = getUpdatedNode();
		framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Node is ' + node)
		recoveryAppLocation = getRecoveryApplicationLocationES( cell, node )
		cluster = getUpdatedCluster()
		framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Cluster is ' + cluster)
		previousVersion = getPropertyValue( 'wbi.upgrade.previous.version' )
		framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Previous version is ' + previousVersion)
		#add for delete the nw wpsFEMgr_6.2.0 that is installed during creating standaone profile
		nodeType = getPropertyValue( 'wbi.upgrade.node.type' )
		if( nodeType == "unmanaged" ):
			if( checkApplicationExist( recoveryApplicationName ) == 1 ):
				try:
					AdminApp.uninstall( recoveryApplicationName )
				except:
					framework.log('error', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Error occurred when trying to uninstall the application')
				else:
					AdminConfig.save()
		#2.2 If node update is performed
		if( cluster == '' ):
			serverList = getServerList( cell, node )
			framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Server list is ' + ''.join(serverList))
			for server in serverList:
				framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Server is ' + server)
				nodeID = AdminConfig.getid( '/Cell:' + cell + '/Node:' + node )
				framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Node id is ' + nodeID)
				serverID = AdminConfig.getid( '/Cell:' + cell + '/Node:' + node + '/Server:' + server )
				framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Server id is ' + serverID)
				#remove static MBean for adapting wps61
				if( isSCAConfigured( node, server, '' ) ):
					framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'SCA is configured on the server')
					scaBusName = 'SCA.SYSTEM.' + cell + '.Bus'
					framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Sca bus name is ' + scaBusName)
					queueID = 'WBI.FailedEvent.' + node + '.' + server
					framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Queue id is ' + queueID)
					destination = findDestination( scaBusName, queueID )
					if( destination == '' ):
						framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'The destination ' + queueID + ' do not exist')
						remoteME = lookupMELocation( node, server, '' )
						meNode = node
						meServer = server
						meCluster = cluster
						if( remoteME != '' ):
							meNode = parseNode( remoteME )
							meServer = parseServer( remoteME )
							meCluster = parseCluster( remoteME )
						else:
							meNode = node
							meServer = server
							meCluster = cluster
						if( meCluster == '' ):
							if( createSIBDestination( scaBusName, meNode, meServer, '', queueID ) == 0 ):
								continue
						else:
							if( createSIBDestination( scaBusName, '', '', meCluster, queueID ) == 0 ):
								continue
					activationSpecName = 'failedevent_AS'
					activationSpecJndiName = 'manualrecovery'
					messageListenerType = 'com.ibm.wsspi.sib.ra.SibRaMessageListener'
					messageSelector = '(SI_CorrelationID IS NULL) AND (user.scaStoredMsgId IS NULL)'
					if( nodeType == 'unmanaged' ):
						if( createASForStandAlone( activationSpecName, activationSpecJndiName, messageListenerType, scaBusName, queueID, messageSelector ) == 0 ):
							continue
					elif( nodeType == 'managed' ):
						if( createASForNDServer( activationSpecName, activationSpecJndiName, messageListenerType, scaBusName, queueID, messageSelector, node, server, '' ) == 0 ):
							continue
					if( installApplicationES( recoveryApplicationName, recoveryAppLocation, cell, node, server ) == 0 ):
						continue
					if( updateAppDestinations( cell, node, server, '', '' ) == 0 ):
						continue

					AdminConfig.save()
					framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Upgrade of node and server successful')

		#2.3 If cluster update is performed
		else:
			#check whether the cluster has the wpsFEMgr or wpsFEmgr_6.0.2.0 mapping. If the cluster has the mapping ,delete the old mapping 
			if( isSCAConfigured( '', '', cluster ) ):
				if ( isMessagingCluster( cluster ) == 0 ):
					framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Cluster has SCA configured and is not a messaging cluster')
					scaBusName = 'SCA.SYSTEM.' + cell + '.Bus'
					queueID = 'WBI.FailedEvent.' + cluster
					destination = findDestination( scaBusName, queueID )
					clusterID = AdminConfig.getid( '/Cell:' + cell + '/ServerCluster:' + cluster )
					remoteME = lookupMELocation( '', '', cluster )
					meNode = ''
					meServer = ''
					meCluster = cluster
					framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Cluster is ' + cluster)
					if( remoteME != '' ):
						meNode = parseNode( remoteME )
						meServer = parseServer( remoteME )
						meCluster = parseCluster( remoteME )
					if( meCluster == '' ):
						if( createSIBDestination( scaBusName, meNode, meServer, '', queueID ) == 0 ):
							framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Exiting 0')
							return 0
					else:
						if( createSIBDestination( scaBusName, '', '', meCluster, queueID ) == 0 ):
							framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Exiting 0')
							return 0
					activationSpecName = 'failedevent_AS'
					activationSpecJndiName = 'manualrecovery'
					messageListenerType = 'com.ibm.wsspi.sib.ra.SibRaMessageListener'
					messageListenerType = 'com.ibm.wsspi.sib.ra.SibRaMessageListener'
					messageSelector = '(SI_CorrelationID IS NULL) AND (user.scaStoredMsgId IS NULL)'
					if( createASForNDServer( activationSpecName, activationSpecJndiName, messageListenerType, scaBusName, queueID, messageSelector, '', '', cluster ) == 0 ):
						framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Exiting 0')
						return 0
					if( installApplicationClusterES( recoveryApplicationName, recoveryAppLocation, cell, cluster ) == 0 ):
						framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Exiting 0')
						return 0
					if( updateAppDestinations( cell, '', '', cluster, meCluster ) == 0 ):
						framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Exiting 0')
						return 0
					AdminConfig.save()
					framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Cluster upgraded successfully')
				else:
					framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Cluster is a messaging cluster, Recovery will not be configured')
			else:
				framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Cluster is not SCA-enabled, Recovery will not be configured')
	# 3. Otherwise update not required, do nothing
	else:
		framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Nothing needs to be done for Recovery component')
		framework.log('info', 'ESBRecoveryUpgrade', 'upgradeESBProfile', 'Exiting')
